/*
 *
 *  Program For Sending Test Message
 *         -- Kazuyuki Tasaka --
 *         (kazuyu-t@is.aist-nara.ac.jp)
 */

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <termios.h>
#include <sys/signal.h>
#include <unistd.h>
#include <fcntl.h>

#define CR '\r'

#define _POSIX_SOURCE 1 
#define BAUDRATE B9600

int fd;
struct termios oldtio;

int open_serial_port(char *modem_dev);
int put_serial_char(unsigned char c);
int put_serial_string(char *s);
int get_serial_char();
void close_serial_port(void);

int main(int argc,char **argv){

  char rs_rbuffer[16];
  char rs_sbuffer[16];
  int c;
  int i=0;
  int wide_num=0;

  char *serial_dev;

  if(argc==3 ){
    serial_dev=argv[1];
    wide_num = atoi(argv[2]);
  }
  else{
    fprintf(stderr,"usage: tagred [serial_dev] [wide-num]\n");
    exit(1);
  }

  //  while(1){
  memset(rs_rbuffer,'\0',16);
  memset(rs_sbuffer,'\0',16);
  sprintf(rs_sbuffer,"3211008000%04d%c",wide_num,CR);

  printf("Test Message Sending...\n");
  open_serial_port(serial_dev);

  put_serial_string(rs_sbuffer);
  printf("     Message is %s \n",rs_sbuffer);
  

  while(1){
    c=0;
    c=get_serial_char();

    rs_rbuffer[i] = c;
    i++;

    if( c == CR){
      printf("ACK Message Received\n");
      printf("     Message(len %d) is %s\n",strlen(rs_rbuffer),rs_rbuffer);

      memset(rs_rbuffer,'\0',16);
      i = 0;
      break;
    }

    if(i>1023)
      break;
  }

  //  }
  close_serial_port();


  exit(0);
}

/*----- Open serial port method -----*/
int open_serial_port(char *modem_dev){
  struct termios newtio;

  fd=open(modem_dev,O_RDWR | O_NOCTTY | O_NONBLOCK);
  if(fd < 0){
    perror(modem_dev);
    exit(-1);
  }

  sleep(0.5);

//  fcntl(fd, F_SETOWN, getpid());

  fcntl(fd, F_SETFL, FASYNC);

  tcgetattr(fd,&oldtio);

  newtio.c_iflag = 0;
  newtio.c_oflag = 0;
  newtio.c_cflag = 0;
  newtio.c_lflag = 0;
  bzero(newtio.c_cc,sizeof(newtio.c_cc));

  cfsetispeed(&newtio,B9600);
  cfsetospeed(&newtio,B9600);
  newtio.c_cflag = CS8 | PARENB | CLOCAL | CREAD | CRTSCTS;
  newtio.c_iflag = INPCK;
  newtio.c_oflag = 0;
  newtio.c_lflag = 0;
  newtio.c_cc[VTIME] = 0;
  newtio.c_cc[VMIN] = 1;
  tcflush(fd,TCIFLUSH);
  tcsetattr(fd,TCSANOW,&newtio);

  return 0;
}

/*----- Close serial port method -----*/
void close_serial_port(void){

  /* Restore the old port settings */
  tcsetattr(fd,TCSANOW,&oldtio);
  close(fd);
}


/*----- Output a character to current serial port -----*/
int put_serial_char(unsigned char c){
  if(write(fd,&c,1) != 1)
    return -1;
  return 0;
}

/*----- Output a string to current serial port -----*/
int put_serial_string(char *s){
  if(write(fd,s,strlen(s)) != 1)
    return -1;
  return 0;
}

/*----- Get a character from current serial port -----*/
int get_serial_char(){
  char c;
  int res;
  res = read(fd,(char *)&c,1);
  return c;
}


